@@ -26,6 +26,7 @@ NOTHING IN THIS FILE HAS BACKWARDS COMPATIBILITY GUARANTEES.
 """
 
 # pytype: skip-file
+# pylint: disable=wrong-import-order, wrong-import-position
 
 import datetime
 import decimal
@@ -45,7 +46,6 @@ from typing import Union
 
 import fastavro
 import numpy as np
-import regex
 
 import apache_beam
 from apache_beam import coders
@@ -53,12 +53,12 @@ from apache_beam.internal.gcp import auth
 from apache_beam.internal.gcp.json_value import from_json_value
 from apache_beam.internal.http_client import get_new_http
 from apache_beam.internal.metrics.metric import MetricLogger
-from apache_beam.internal.metrics.metric import Metrics
 from apache_beam.internal.metrics.metric import ServiceCallMetric
 from apache_beam.io.gcp import bigquery_avro_tools
 from apache_beam.io.gcp import resource_identifiers
 from apache_beam.io.gcp.internal.clients import bigquery
 from apache_beam.metrics import monitoring_infos
+from apache_beam.metrics.metric import Metrics
 from apache_beam.options import value_provider
 from apache_beam.options.pipeline_options import PipelineOptions
 from apache_beam.transforms import DoFn
@@ -68,14 +68,16 @@ from apache_beam.utils import retry
 from apache_beam.utils.histogram import LinearBucket
 
 # Protect against environments where bigquery library is not available.
-# pylint: disable=wrong-import-order, wrong-import-position
 try:
+  import regex
+  from apitools.base.py.exceptions import HttpError
+  from apitools.base.py.exceptions import HttpForbiddenError
   from apitools.base.py.transfer import Upload
-  from apitools.base.py.exceptions import HttpError, HttpForbiddenError
-  from google.api_core.exceptions import ClientError, GoogleAPICallError
   from google.api_core.client_info import ClientInfo
+  from google.api_core.exceptions import ClientError
+  from google.api_core.exceptions import GoogleAPICallError
   from google.cloud import bigquery as gcp_bigquery
-except ImportError:
+except Exception:
   gcp_bigquery = None
   pass
 
@@ -121,6 +123,7 @@ BIGQUERY_TYPE_TO_PYTHON_TYPE = {
     "FLOAT": np.float64,
     "NUMERIC": decimal.Decimal,
     "TIMESTAMP": apache_beam.utils.timestamp.Timestamp,
+    "GEOGRAPHY": str,
 }
 
 
@@ -333,6 +336,10 @@ def _build_filter_from_labels(labels):
   return filter_str
 
 
+def _build_dataset_encryption_config(kms_key):
+  return bigquery.EncryptionConfiguration(kmsKeyName=kms_key)
+
+
 class BigQueryWrapper(object):
   """BigQuery client wrapper with utilities for querying.
 
@@ -414,7 +421,7 @@ class BigQueryWrapper(object):
 
   def _get_temp_table_project(self, fallback_project_id):
     """Returns the project ID for temporary table operations.
-    
+
     If temp_table_ref exists, returns its projectId.
     Otherwise, returns the fallback_project_id.
     """
@@ -835,7 +842,7 @@ class BigQueryWrapper(object):
       num_retries=MAX_RETRIES,
       retry_filter=retry.retry_on_server_errors_and_timeout_filter)
   def get_or_create_dataset(
-      self, project_id, dataset_id, location=None, labels=None):
+      self, project_id, dataset_id, location=None, labels=None, kms_key=None):
     # Check if dataset already exists otherwise create it
     try:
       dataset = self.client.datasets.Get(
@@ -858,6 +865,9 @@ class BigQueryWrapper(object):
           dataset.location = location
         if labels is not None:
           dataset.labels = _build_dataset_labels(labels)
+        if kms_key is not None:
+          dataset.defaultEncryptionConfiguration = (
+              _build_dataset_encryption_config(kms_key))
         request = bigquery.BigqueryDatasetsInsertRequest(
             projectId=project_id, dataset=dataset)
         response = self.client.datasets.Insert(request)
@@ -929,9 +939,14 @@ class BigQueryWrapper(object):
   @retry.with_exponential_backoff(
       num_retries=MAX_RETRIES,
       retry_filter=retry.retry_on_server_errors_and_timeout_filter)
-  def create_temporary_dataset(self, project_id, location, labels=None):
+  def create_temporary_dataset(
+      self, project_id, location, labels=None, kms_key=None):
     self.get_or_create_dataset(
-        project_id, self.temp_dataset_id, location=location, labels=labels)
+        project_id,
+        self.temp_dataset_id,
+        location=location,
+        labels=labels,
+        kms_key=kms_key)
 
     if (project_id is not None and not self.is_user_configured_dataset() and
         not self.created_temp_dataset):
