@@ -35,7 +35,6 @@ from typing import Tuple
 from typing import TypeVar
 from typing import Union
 
-import redis
 from google.api_core.exceptions import TooManyRequests
 
 import apache_beam as beam
@@ -46,6 +45,11 @@ from apache_beam.metrics import Metrics
 from apache_beam.transforms.util import BatchElements
 from apache_beam.utils import retry
 
+try:
+  import redis
+except ImportError:
+  redis = None
+
 RequestT = TypeVar('RequestT')
 ResponseT = TypeVar('ResponseT')
 
@@ -689,6 +693,11 @@ class RedisCache(Cache):
     self._kwargs = kwargs if kwargs else {}
     self._source_caller = None
 
+    if redis is None:
+      raise ImportError(
+          'Failed to import redis. You can ensure it is '
+          'installed by installing the redis beam extra')
+
   def get_read(self):
     """get_read returns a PTransform for reading from the cache."""
     ensure_coders_exist(self._request_coder)
